package Crypt::Mode;

use strict;
use warnings;
our $VERSION = '0.089';

### not used

1;

=pod

=head1 NAME

Crypt::Mode - [internal only]

=head1 SYNOPSIS

Do not use this module directly.

Use a concrete block-mode module instead.

=head1 SEE ALSO

=over

=item * L<CryptX>

=item * L<Crypt::Mode::CBC>, L<Crypt::Mode::CFB>, L<Crypt::Mode::CTR>, L<Crypt::Mode::ECB>, L<Crypt::Mode::OFB>

=item * L<Crypt::AuthEnc::ChaCha20Poly1305>, L<Crypt::AuthEnc::GCM>

=back

=cut
